@@ -1,6 +1,7 @@
 import requests
 import logging
 from expai.expai_model import ExpaiModelExplainer
+from expai.utils import generate_response
 
 
 class ExpaiProject:
@@ -19,20 +20,41 @@ class ExpaiProject:
     ###      MODELS       ###
     #########################
 
-    def model_list(self):
+    def model_list(self, search_by: str = None, exact_search: str = None):
         """
         List all models within the project
         :return: list of models
         """
-        response = self.sess.request("GET", self.server_name + "/api/projects/{}/model/list".format(self.project_id),
-                                    headers=self.headers)
-        if response.ok:
-            return response.json()['models']
-        else:
-            logging.error("There was an error listing your models. Please, check the details below. \n {}".format(
-                response.json()['message']))
+
+        if exact_search is not None:
+            response = self.sess.request("GET",
+                                         self.server_name + "/api/projects/{}/model/list/contains/{}".format(
+                                             self.project_id, exact_search),
+                                         headers=self.headers)
+
+            if 'models' in response.json().keys():
+                for model in response.json()['models']:
+                    if model['model_name_des'] == exact_search:
+                        return model
+            logging.error(
+                "We could not find a sample with this exact name. Try again or consider using search_by parameter to find similar samples")
             return None
 
+        elif search_by is not None:
+            response = self.sess.request("GET",
+                                         self.server_name + "/api/projects/{}/model/list/contains/{}".format(
+                                             self.project_id, search_by),
+                                         headers=self.headers)
+            return generate_response(response, 'models')
+
+        else:
+            response = self.sess.request("GET",
+                                         self.server_name + "/api/projects/{}/model/list".format(self.project_id),
+                                         headers=self.headers)
+
+            return generate_response(response, 'models')
+
+
     def create_model(self, model_path: str = None, is_supervised: bool = True, model_name: str = None,
                      model_type: str = None, model_library: str = None, model_objective: str = None,
                      model_prediction_type: str = None):
@@ -65,11 +87,9 @@ class ExpaiProject:
                                     data=data, headers=headers, files=files)
 
         if response.ok:
-            return True
+            return "Model successfully created with id {}".format(response.json()['model_id'])
         else:
-            logging.error("There was an error creating the model. Please, check the details below. \n {}".format(
-                response.json()['message']))
-            return None
+            return generate_response(response)
 
     def delete_model(self, model_name: str = None, model_id: str = None):
         """
@@ -88,11 +108,9 @@ class ExpaiProject:
                                     headers=self.headers)
 
         if response.ok:
-            return True
+            return "Model successfully deleted"
         else:
-            logging.error("There was an error deleting the model. Please, check the details below. \n {}".format(
-                response.json()['message']))
-            return None
+            return generate_response(response)
 
     def get_model_explainer(self, model_name: str = None, model_id: str = None):
         """
@@ -125,20 +143,36 @@ class ExpaiProject:
     ###      SAMPLES      ###
     #########################
 
-    def sample_list(self):
+    def sample_list(self, search_by: str = None, exact_search: str = None):
         """
         List all samples associated with this project
         :return: list of samples
         """
-        response = self.sess.request("GET", self.server_name + "/api/projects/{}/sample/list".format(self.project_id),
+
+        if exact_search is not None:
+            response = self.sess.request("GET",
+                                         self.server_name + "/api/projects/{}/sample/list/contains/{}".format(self.project_id, exact_search),
+                                         headers=self.headers)
+
+            if 'samples' in response.json().keys():
+                for sample in response.json()['samples']:
+                    if sample['sample_name_des'] == exact_search:
+                        return sample
+            logging.error("We could not find a sample with this exact name. Try again or consider using search_by parameter to find similar samples")
+            return None
+
+        elif search_by is not None:
+            response = self.sess.request("GET",
+                                         self.server_name + "/api/projects/{}/sample/list/contains/{}".format(
+                                             self.project_id, search_by),
+                                         headers=self.headers)
+            return generate_response(response, 'samples')
+
+        else:
+            response = self.sess.request("GET", self.server_name + "/api/projects/{}/sample/list".format(self.project_id),
                                     headers=self.headers)
 
-        if response.ok:
-            return response.json()['samples']
-        else:
-            logging.error("There was an error listing your samples. Please, check the details below. \n {}".format(
-                response.json()['message']))
-            return None
+            return generate_response(response, 'samples')
 
     def create_sample(self, sample_path: str = None, sample_name: str = None, sample_separator: str = None, sample_target_col: str = None,
                       sample_encoding: str = 'utf-8', is_display = None, reference_sample_id: str = None, reference_sample_name: str = None):
@@ -155,7 +189,7 @@ class ExpaiProject:
         assert sample_path is not None, "To create a sample you must specify the path to the stored file"
         assert sample_separator is not None, "Please, specify the file separator"
 
-        if is_display is not None:
+        if is_display is True:
             assert reference_sample_id is not None or reference_sample_name is not None, "Please, specify the original sample id or name for this display"
 
             if reference_sample_id is None:
@@ -169,7 +203,7 @@ class ExpaiProject:
                 "sample_file_separator_des": sample_separator,
                 "sample_file_encoding_des": sample_encoding,
                 "sample_target_col": sample_target_col,
-                "is_display": 1 if is_display is not None else None,
+                "is_display": 1 if is_display is True else 0,
                 "original_sample_id": reference_sample_id}
 
         files = {'sample_file': open(sample_path, 'r', encoding=sample_encoding)}
@@ -180,11 +214,9 @@ class ExpaiProject:
                                     data=data, headers=headers, files=files)
 
         if response.ok:
-            return True
+            return "Sample successfully created with id {}".format(response.json()['sample_id'])
         else:
-            logging.error("There was an error creating the model. Please, check the details below. \n {}".format(
-                response.json()['message']))
-            return None
+            return generate_response(response)
 
     def delete_sample(self, sample_name: str = None, sample_id: str = None):
         """
@@ -208,11 +240,9 @@ class ExpaiProject:
                                     headers=self.headers)
 
         if response.ok:
-            return True
+            return "Sample successfully deleted"
         else:
-            logging.error("There was an error deleting the sample. Please, check the details below. \n {}".format(
-                response.json()['message']))
-            return None
+            return generate_response(response)
 
     def _get_sample_id_from_name(self, sample_name: str = None):
 
