@@ -117,10 +117,8 @@ class ExpaiLogin:
             return None
 
     def _get_project_id_from_name(self, project_name: str = None):
-        project_list = self.search_project(search_by=project_name)
-
-        for project in project_list:
-            if project['project_name_des'] == project_name:
-                return project['project_id']
+        project = self.project_list(exact_search=project_name)
+        if project is not None:
+            return project['project_id']
 
         return None
